<?php
/**
 * TemplateGlue.inc.php
 * Code by Strangecode :: www.strangecode.com :: This document contains copyrighted information
 */

/**
 * Print URL to download file with BBEdit/Interarchy. "USER" must be a pre-set 
 * environment variable. Files must reside within and be relative to env "DOCUMENT_ROOT".
 *
 * @access  public
 *
 * @param   mixed   $file   File path relative to DOCUMENT_ROOT (i.e. '/_templates/index.ihtml').
 *
 * @return  string  <a> with file basename as text.
 */
function bbftp($file)
{
    return sprintf('<a href="bbftp://%s@%s:%s%s" title="%s">%s</a>', $_SERVER['USER'], $_SERVER['HTTP_HOST'], SITE_BASE, $file, $file, basename($file));
}

/**
 * Prints an image tag for image specified in $src. 
 *
 * @param  string $src     File name of the image, including dir and file extension.
 * @param  string $alt     Alt tag text.
 * @param  string $extra   Additional attributes.
 */
function oImg($src, $alt='', $extra='')
{
    $filepath = preg_match('!://!', $src) ? $src : getenv('DOCUMENT_ROOT') . $src;
    
    if (false === ($gis = @getimagesize($filepath)) || preg_match('/width|height/', $extra)) {
        $image_size = '';
    } else {
        $image_size = $gis[3];
    }
    
    return sprintf('<img src="%s" %s alt="%s" %s />',
        $src,
        $image_size,
        oTxt($alt),
        $extra
    );
}
function printImg($src, $alt='', $extra='')
{
    echo oImg($src, $alt, $extra);
}

/**
 * Finds the values of an enumeration or set column of a MySQL database, returning them in an array.
 * Use this to generate a pull-down menu of options or to validate the existance
 * of options. (Quinn 10 Feb 2001) 
 *
 * @param  string $db_table   database table to lookup
 * @param  string $db_col     database column to lookup
 *
 * @return array    Array of the set/enum values on success, false on failure.
 */
function getSetEnumFieldValues($db_table, $db_col)
{
	$qid = dbQuery("SHOW COLUMNS FROM $db_table LIKE '$db_col'",false);
		
	$row = mysql_fetch_row($qid);
	if (preg_match('/^enum|^set/i', $row[1]) && preg_match_all("/'([^']*)'/", $row[1], $enum)) {
		return $enum[1];
	} else {
		return false;
	}
}

/**
 * Prints option fields for a select form. Works only with enum or set
 * data types in table columns. 
 *
 * @param  string $db_table   database table to lookup
 * @param  string $db_col     database column to lookup
 */
function printSetSelectForm($db_table, $db_col, $preselected, $blank=false)
{
	$values = getSetEnumFieldValues($db_table, $db_col);
	if ($values === false) {
        ?><option value=""><?php echo _("n/a"); ?></option>
        <?php
        return false;
	}
	if (true === $blank) {
		$selected = ($preselected == '') ? ' selected' : '';
        ?><option value=""<?php echo $selected; ?>>&nbsp;</option>
        <?php
	}
	// When the 'blank' value needs a specific key->val pair.
	if (is_array($blank)) {
        foreach ($blank as $key=>$val) {
            $selected = ($preselected == $val) ? ' selected="selected"' : '';
            ?><option value="<?php echo $key; ?>"<?php echo $selected; ?>><?php echo oTxt($val); ?></option>
            <?php
        }
	}
	foreach ($values as $item) {
		$selected = ($item == $preselected) ? ' selected' : '';
		?><option value="<?php echo $item; ?>"<?php echo $selected; ?>><?php echo oTxt($item); ?></option>
		<?php
	}
}

/**
 * Prints checkbox fields. Works only with enum or set
 * data types in table columns. 
 *
 * @param  string $db_table      database table to lookup
 * @param  string $db_col        database column to lookup
 * @param  array  $preselected   array of preselected values (matching the values in $db_col)
 * @param  int    $columns       number of table columns to print
 * @param  int    $flag          set to 'allone' for name of input fields to all be the same of a multidimentional array.
 */
function printSetCheckboxes($db_table, $db_col, $preselected, $columns=1, $flag=null)
{	
    ?>
    <table>
        <tr>
    <?php
    // Sometimes preselected comes as a comma list.
    if (!is_array($preselected)) {
        $preselected = explode(',', $preselected);
    }
    
    // Checkbox POST data has the primary data in the keys, and 'on' as the values.
    // Here we assume in all the values of an array are 'on' that we can find the data
    // in the array keys.
    $ps_value_count = array_count_values($preselected);
    if ($ps_value_count['on'] == sizeof($preselected)) {
        $preselected = array_keys($preselected);
    }
    
    // Retreive values of a Set or ENUM database column. 
	$values = getSetEnumFieldValues($db_table, $db_col);
	
    // Initialize the HTML table generation vars.
    $num_cells = sizeof($values) - 1;
    $num_rows = floor($num_cells / $columns);
    $cols_lastrow = $num_cells % $columns;
    $row_cnt = 0;
    $col_cnt = 0;
	foreach ($values as $item) {
        if ($col_cnt == $columns) {
            // Begin a new row.
            ?></tr><tr><?php
            $col_cnt = 0;
            $row_cnt++;
        }
        if ($col_cnt < $cols_lastrow) {
            $lastrow_add = $col_cnt;
        } else {
            $lastrow_add = $cols_lastrow;
        }
        $curr = $num_rows * $col_cnt + $lastrow_add + $row_cnt;
        $col_cnt++;
        
        // Look for preselected value.
        if (in_array($item, $preselected)) {
            $checked = ' checked="checked"';
        } else {
            $checked = '';
        }
        if ('allone' == $flag) {
            // Print a cell with multidimentioal array checkboxes.
            ?><td><input type="checkbox" name="dbcol[<?php echo $db_col; ?>][<?php echo $item; ?>]"<?php echo $checked; ?> />&nbsp;<?php echo oTxt($item); ?>&nbsp;</td>
            <?php
         } else {
            // Print a cell with basic named checkboxes.
            ?><td><input type="checkbox" name="<?php echo $db_col; ?>[<?php echo $item; ?>]"<?php echo $checked; ?> />&nbsp;<?php echo oTxt($item); ?>&nbsp;</td>
            <?php
        }
    }
    if ($col_cnt < $columns) {
        // This last cell must expand to fill the last blank cells.
        ?><td colspan="<?php echo $columns - $col_cnt; ?>">&nbsp;</td><?php
    }
    ?>
        </tr>
    </table><?php
}

/**
 * Prints radio select fields. Works only with enum or set
 * data types in table columns. 
 *
 * @param  string $db_table      database table to lookup
 * @param  string $db_col        database column to lookup
 * @param  array  $preselected   array of preselected values (matching the values in $db_col)
 * @param  int    $columns       number of table columns to print
 * @param  int    $flag          set to 'allone' for name of input fields to all be the same of a multidimentional array.
 */
function printSetRadios($db_table, $db_col, $preselected, $columns=1, $flag=null)
{	
    ?>
    <table>
        <tr>
    <?php
    // Sometimes preselected comes as a comma list.
    if (!is_array($preselected)) {
        $preselected = explode(',', $preselected);
    }
    
    // Checkbox POST data has the primary data in the keys, and 'on' as the values.
    // Here we assume in all the values of an array are 'on' that we can find the data
    // in the array keys.
    $ps_value_count = array_count_values($preselected);
    if ($ps_value_count['on'] == sizeof($preselected)) {
        $preselected = array_keys($preselected);
    }
    
    // Retreive values of a Set or ENUM database column. 
	$values = getSetEnumFieldValues($db_table, $db_col);
	
    // Initialize the HTML table generation vars.
    $num_cells = sizeof($values) - 1;
    $num_rows = floor($num_cells / $columns);
    $cols_lastrow = $num_cells % $columns;
    $row_cnt = 0;
    $col_cnt = 0;
	foreach ($values as $item) {
        if ($col_cnt == $columns) {
            // Begin a new row.
            ?></tr><tr><?php
            $col_cnt = 0;
            $row_cnt++;
        }
        if ($col_cnt < $cols_lastrow) {
            $lastrow_add = $col_cnt;
        } else {
            $lastrow_add = $cols_lastrow;
        }
        $curr = $num_rows * $col_cnt + $lastrow_add + $row_cnt;
        $col_cnt++;
        
        // Look for preselected value.
        if (in_array($item, $preselected)) {
            $checked = ' checked="checked"';
        } else {
            $checked = '';
        }
        // Print a cell with basic named checkboxes.
        ?><td><input type="radio" name="<?php echo $db_col; ?>" value="<?php echo $item ?>"<?php echo $checked; ?> />&nbsp;<?php echo oTxt($item); ?>&nbsp;</td>
        <?php
    }
    if ($col_cnt < $columns) {
        // This last cell must expand to fill the last blank cells.
        ?><td colspan="<?php echo $columns - $col_cnt; ?>">&nbsp;</td><?php
    }
    ?>
        </tr>
    </table><?php
}

/**
 * Prints a pulldown menu containing the specified values and keys of a table. 
 *
 * @param  string $db_table         database table to lookup
 * @param  string $key_column       column containing the human-readable titles for the select menu
 * @param  string $val_column       column containing the computer values for the select menu
 * @param  string $preselected      the currently selected value of the menu. compared to the $val_column
 * @param  bool   $blank            leave one blank at the top?
 * @param  string $extra_clause     SQL exclude cluase. Something like "WHERE girls != 'buckteeth'"
 */
function printSelectForm($db_table, $key_column, $val_column, $preselected, $blank=false, $extra_clause='')
{
    // Sometimes preselected comes as a comma list.
    if (!is_array($preselected)) {
        $preselected = array($preselected);
    }

    // Print a blank first option.
	if (true === $blank) {
		$selected = in_array('', $preselected) ? ' selected="selected"' : '';
        ?><option value=""<?php echo $selected; ?>>&nbsp;</option>
        <?php
	}
	
	// When the 'blank' value needs a specific key->val pair.
	if (is_array($blank)) {
        foreach ($blank as $key=>$val) {
            $selected = in_array($val, $preselected) ? ' selected="selected"' : '';
            ?><option value="<?php echo $key; ?>"<?php echo $selected; ?>><?php echo oTxt($val); ?></option>
            <?php
        }
	}
	$qid = dbQuery("SELECT $key_column, $val_column FROM $db_table $extra_clause",false);
	while ($row = mysql_fetch_assoc($qid)) {
		$selected = in_array($row[$val_column], $preselected) ? ' selected="selected"' : '';
		?><option value="<?php echo $row[$val_column]; ?>"<?php echo $selected; ?>><?php echo oTxt($row[$key_column]); ?></option><?php
	}
}

/**
 * Prints checkbox fields. Works only with enum or set
 * data types in table columns. 
 *
 * @param  string $db_table         database table to lookup
 * @param  string $key_column       column containing the human-readable titles for the select menu
 * @param  string $val_column       column containing the computer values for the select menu
 * @param  string $preselected      the currently selected value of the menu. compared to the $val_column
 * @param  int    $columns          number of table columns to print
 * @param  int    $extra_clause     extra sql to send at end of select statement.
 * @param  int    $vert_columns     display checkboxes in vertical orientation first.
 */
function printDBCheckboxes($db_table, $key_column, $val_column, $preselected, $columns=1, $extra_clause='', $vert_columns=true)
{
    // Sometimes preselected comes as a comma list.
    if (!is_array($preselected)) {
        $preselected = explode(',', $preselected);
    }
    
    // Checkbox POST data has the primary data in the keys, and 'on' as the values.
    // Here we assume in all the values of an array are 'on' that we can find the data
    // in the array keys.
    $ps_value_count = array_count_values($preselected);
    if ($ps_value_count['on'] == sizeof($preselected)) {
        $preselected = array_keys($preselected);
    }
	
	$qid = dbQuery("SELECT $key_column, $val_column FROM $db_table $extra_clause",false);
	while ($row = mysql_fetch_assoc($qid)) {
	    $values[] = $row;
	}
	
	// Rearrange array so sort is in vertical columns.
// 	if ($vert_columns) {
// 	    $per_col = ceil(sizeof($values) / $columns);
// 	    $columns = ceil(sizeof($values) / $per_col);
// 	    $curr_row = 0;
// 	    $curr_col = 0;
// 	    $pos = 0;
//         foreach ($values as $k=>$v) {
//             $pos = $curr_row * $columns + $curr_col;
//             if ($curr_row <= $per_col) {
//                 $curr_row++;
//             } else {
//                 $curr_row = 0;
//                 $curr_col++;
//             }
// //             echo '<li>' . $pos . '-' . $v[$key_column];
//             $new_values[$pos] = $v;
//         }
//         $values = $new_values;
//         ksort($values);
// 	}

	if (empty($values)) {
	    return false;
	}
	
    // Initialize the HTML table generation vars.
    $num_cells = sizeof($values) - 1;
    $num_rows = floor($num_cells / $columns);
    $cols_lastrow = $num_cells % $columns;
    $row_cnt = 0;
    $col_cnt = 0;
    ?>
    <table border="0" cellspacing="0" cellpadding="0" class="formcheckboxes">
        <tr>
    <?php
	foreach ($values as $box) {
        if ($col_cnt == $columns) {
            // Begin a new row.
            ?></tr><tr><?php
            $col_cnt = 0;
            $row_cnt++;
        }
        
        if ($col_cnt < $cols_lastrow) {
            $lastrow_add = $col_cnt;
        } else {
            $lastrow_add = $cols_lastrow;
        }
        
        $curr = $num_rows * $col_cnt + $lastrow_add + $row_cnt;
        $col_cnt++;
        
        // Look for preselected value.
        if (in_array($box[$val_column], $preselected)) {
            $checked = ' checked="checked"';
        } else {
            $checked = '';
        }
        
        // Print a cell with basic named checkboxes.
        ?><td><input type="checkbox" name="<?php echo $val_column; ?>[<?php echo $box[$val_column]; ?>]"<?php echo $checked; ?> /></td><td><?php echo oTxt($box[$key_column]); ?></td>
        <?php
    }
    
    if ($col_cnt < $columns) {
        // This last cell must expand to fill the last blank cells.
        ?><td colspan="<?php echo $columns - $col_cnt; ?>">&nbsp;</td><?php
    }
    ?>
        </tr>
    </table><?php
}

?>